@@ -6,7 +6,7 @@ import crypto from 'crypto';
 // 전체 시스템 스키마 통합 정리
 // =============================================================================
 
-const newCurrentVersion = '3.3';
+const newCurrentVersion = '3.4';
 
 // 마이그레이션 버전 관리
 const createMigrationTable = async (dbManager) => {
@@ -83,7 +83,7 @@ const createTables = async (dbManager) => {
       return;
     }
 
-    console.log('데이터베이스 마이그레이션 시작 (버전 ' + newCurrentVersion + ' - PR #265 통합 및 스키마 안정성 개선)...');
+    console.log('데이터베이스 마이그레이션 시작 (버전 ' + newCurrentVersion + ' - LLM 파인튜닝 시스템 추가)...');
 
     await ensureProjectsTable(dbManager);
 
@@ -386,7 +386,9 @@ const createTables = async (dbManager) => {
     // pgvector 확장 생성
     await dbManager.query(`CREATE EXTENSION IF NOT EXISTS vector;`);
 
-    // 문서 임베딩 테이블 (통일된 스키마)
+    // 문서 임베딩 테이블 (기본 테이블 - 검색/관리용)
+    // 주의: 모델별 임베딩 테이블(예: nlpai_lab_kure_v1_document_embeddings)은
+    //      RAG 서비스가 필요 시 자동으로 동적 생성하므로 여기서 정의하지 않음
     await dbManager.query(`
             CREATE TABLE IF NOT EXISTS document_embeddings (
                 id SERIAL PRIMARY KEY,
@@ -402,7 +404,7 @@ const createTables = async (dbManager) => {
                 file_mtime BIGINT,                  -- 파일 수정 시간 (Unix timestamp)
                 metadata JSONB DEFAULT '{}',
                 created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
-                
+
                 -- 복합 유니크 제약조건
                 CONSTRAINT unique_doc_chunk UNIQUE (doc_id, chunk_index)
             );
@@ -653,6 +655,72 @@ const createTables = async (dbManager) => {
             );
         `);
 
+    // 템플릿 카테고리 테이블
+    await dbManager.query(`
+            CREATE TABLE IF NOT EXISTS flowai_template_categories (
+                id VARCHAR(255) PRIMARY KEY,
+                name VARCHAR(255) NOT NULL,
+                description TEXT,
+                icon VARCHAR(50) DEFAULT '📁',
+                color VARCHAR(7) DEFAULT '#666666',
+                order_index INTEGER DEFAULT 999,
+                is_active BOOLEAN DEFAULT true,
+                created_at TIMESTAMP DEFAULT now(),
+                updated_at TIMESTAMP DEFAULT now()
+            );
+        `);
+
+    // 템플릿 테이블
+    await dbManager.query(`
+            CREATE TABLE IF NOT EXISTS flowai_templates (
+                id VARCHAR(255) PRIMARY KEY,
+                name VARCHAR(255) NOT NULL,
+                description TEXT,
+                category_id VARCHAR(255) REFERENCES flowai_template_categories(id) ON DELETE SET NULL,
+                tags TEXT DEFAULT '[]',
+                icon VARCHAR(50) DEFAULT '⚡',
+                difficulty VARCHAR(50) DEFAULT 'beginner',
+                estimated_time INTEGER DEFAULT 5,
+                nodes TEXT DEFAULT '[]',
+                connections TEXT DEFAULT '[]',
+                variables TEXT DEFAULT '[]',
+                preview_image TEXT,
+                preview_description TEXT,
+                author_name VARCHAR(255),
+                author_organization VARCHAR(255),
+                version VARCHAR(50) DEFAULT '1.0.0',
+                is_public BOOLEAN DEFAULT false,
+                usage_count INTEGER DEFAULT 0,
+                rating NUMERIC(3,2) DEFAULT 0.00,
+                user_id INTEGER,
+                created_at TIMESTAMP DEFAULT now(),
+                updated_at TIMESTAMP DEFAULT now()
+            );
+        `);
+
+    // 템플릿 리뷰 테이블
+    await dbManager.query(`
+            CREATE TABLE IF NOT EXISTS flowai_template_reviews (
+                id SERIAL PRIMARY KEY,
+                template_id VARCHAR(255) NOT NULL REFERENCES flowai_templates(id) ON DELETE CASCADE,
+                user_id INTEGER NOT NULL,
+                rating INTEGER CHECK (rating >= 1 AND rating <= 5),
+                comment TEXT,
+                created_at TIMESTAMP DEFAULT now(),
+                updated_at TIMESTAMP DEFAULT now()
+            );
+        `);
+
+    // 템플릿 사용 통계 테이블
+    await dbManager.query(`
+            CREATE TABLE IF NOT EXISTS flowai_template_usage (
+                id SERIAL PRIMARY KEY,
+                template_id VARCHAR(255) NOT NULL REFERENCES flowai_templates(id) ON DELETE CASCADE,
+                user_id INTEGER NOT NULL,
+                used_at TIMESTAMP DEFAULT now()
+            );
+        `);
+
     // =============================================================================
     // 8. AI 학습 및 피드백 시스템
     // =============================================================================
@@ -1243,7 +1311,73 @@ const createTables = async (dbManager) => {
     console.log('건강검진 배치 처리 테이블 생성 완료');
 
     // =============================================================================
-    // 14. 인덱스 생성 (성능 최적화)
+    // 14. LLM 파인튜닝 시스템 (v3.4)
+    // =============================================================================
+
+    // 파인튜닝 데이터셋 테이블
+    await dbManager.query(`
+      CREATE TABLE IF NOT EXISTS finetuning_datasets (
+        id VARCHAR(255) PRIMARY KEY,
+        name VARCHAR(255) NOT NULL,
+        description TEXT,
+        source VARCHAR(50) NOT NULL,
+        file_path TEXT,
+        total_samples INTEGER DEFAULT 0,
+        status VARCHAR(20) DEFAULT 'pending' CHECK (status IN ('pending', 'processing', 'completed', 'failed')),
+        metadata JSONB DEFAULT '{}',
+        created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
+        updated_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
+      );
+    `);
+
+    // 파인튜닝 모델 테이블
+    await dbManager.query(`
+      CREATE TABLE IF NOT EXISTS finetuned_models (
+        id VARCHAR(255) PRIMARY KEY,
+        base_model VARCHAR(255) NOT NULL,
+        dataset_id VARCHAR(255) REFERENCES finetuning_datasets(id) ON DELETE SET NULL,
+        status VARCHAR(50) NOT NULL CHECK (status IN ('training', 'completed', 'failed', 'cancelled')),
+        output_path TEXT,
+        training_config JSONB DEFAULT '{}',
+        metrics JSONB DEFAULT '{}',
+        created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
+        updated_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
+      );
+    `);
+
+    // 파인튜닝 작업 로그 테이블
+    await dbManager.query(`
+      CREATE TABLE IF NOT EXISTS finetuning_jobs (
+        id SERIAL PRIMARY KEY,
+        model_id VARCHAR(255) REFERENCES finetuned_models(id) ON DELETE CASCADE,
+        status VARCHAR(20) DEFAULT 'queued' CHECK (status IN ('queued', 'running', 'completed', 'failed', 'cancelled')),
+        progress INTEGER DEFAULT 0,
+        logs TEXT,
+        error_message TEXT,
+        started_at TIMESTAMP,
+        completed_at TIMESTAMP,
+        created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
+      );
+    `);
+
+    // 인덱스 생성
+    await dbManager.query(`
+      CREATE INDEX IF NOT EXISTS idx_finetuning_datasets_status ON finetuning_datasets(status);
+      CREATE INDEX IF NOT EXISTS idx_finetuning_datasets_created_at ON finetuning_datasets(created_at);
+
+      CREATE INDEX IF NOT EXISTS idx_finetuned_models_status ON finetuned_models(status);
+      CREATE INDEX IF NOT EXISTS idx_finetuned_models_dataset_id ON finetuned_models(dataset_id);
+      CREATE INDEX IF NOT EXISTS idx_finetuned_models_created_at ON finetuned_models(created_at);
+
+      CREATE INDEX IF NOT EXISTS idx_finetuning_jobs_model_id ON finetuning_jobs(model_id);
+      CREATE INDEX IF NOT EXISTS idx_finetuning_jobs_status ON finetuning_jobs(status);
+      CREATE INDEX IF NOT EXISTS idx_finetuning_jobs_created_at ON finetuning_jobs(created_at);
+    `);
+
+    console.log('LLM 파인튜닝 시스템 테이블 생성 완료');
+
+    // =============================================================================
+    // 15. 인덱스 생성 (성능 최적화)
     // =============================================================================
 
     console.log('인덱스 생성 중...');
@@ -1293,8 +1427,31 @@ const createTables = async (dbManager) => {
 
     // 이미지 임베딩 벡터 검색을 위한 인덱스
     await dbManager.query(`
-            CREATE INDEX IF NOT EXISTS idx_document_embeddings_image_vector_cosine 
-            ON document_embeddings USING ivfflat (image_embedding vector_cosine_ops) 
+            CREATE INDEX IF NOT EXISTS idx_document_embeddings_image_vector_cosine
+            ON document_embeddings USING ivfflat (image_embedding vector_cosine_ops)
+            WITH (lists = 100);
+        `);
+
+    // RAG 전용 임베딩 테이블 인덱스
+    await dbManager.query(`
+            CREATE INDEX IF NOT EXISTS idx_nlpai_lab_kure_v1_document_embeddings_doc_id ON nlpai_lab_kure_v1_document_embeddings(doc_id);
+            CREATE INDEX IF NOT EXISTS idx_nlpai_lab_kure_v1_document_embeddings_filename ON nlpai_lab_kure_v1_document_embeddings(filename);
+            CREATE INDEX IF NOT EXISTS idx_nlpai_lab_kure_v1_document_embeddings_chunk_index ON nlpai_lab_kure_v1_document_embeddings(chunk_index);
+            CREATE INDEX IF NOT EXISTS idx_nlpai_lab_kure_v1_document_embeddings_user_id ON nlpai_lab_kure_v1_document_embeddings(user_id);
+            CREATE INDEX IF NOT EXISTS idx_nlpai_lab_kure_v1_document_embeddings_source ON nlpai_lab_kure_v1_document_embeddings(source);
+            CREATE INDEX IF NOT EXISTS idx_nlpai_lab_kure_v1_document_embeddings_created_at ON nlpai_lab_kure_v1_document_embeddings(created_at);
+            CREATE INDEX IF NOT EXISTS idx_nlpai_lab_kure_v1_document_embeddings_has_image ON nlpai_lab_kure_v1_document_embeddings((image_embedding IS NOT NULL));
+            CREATE INDEX IF NOT EXISTS idx_nlpai_lab_kure_v1_document_embeddings_is_embedding ON nlpai_lab_kure_v1_document_embeddings(is_embedding);
+        `);
+
+    // RAG 전용 벡터 인덱스
+    await dbManager.query(`
+            CREATE INDEX IF NOT EXISTS idx_nlpai_lab_kure_v1_document_embeddings_vector_cosine
+            ON nlpai_lab_kure_v1_document_embeddings USING ivfflat (embedding vector_cosine_ops)
+            WITH (lists = 100);
+
+            CREATE INDEX IF NOT EXISTS idx_nlpai_lab_kure_v1_document_embeddings_image_vector_cosine
+            ON nlpai_lab_kure_v1_document_embeddings USING ivfflat (image_embedding vector_cosine_ops)
             WITH (lists = 100);
         `);
 
@@ -1348,6 +1505,28 @@ const createTables = async (dbManager) => {
             CREATE INDEX IF NOT EXISTS idx_flowai_executions_workflow ON flowai_executions(workflow_id);
             CREATE INDEX IF NOT EXISTS idx_flowai_executions_user ON flowai_executions(user_id);
             CREATE INDEX IF NOT EXISTS idx_flowai_executions_status ON flowai_executions(status);
+
+            -- 템플릿 카테고리 인덱스
+            CREATE INDEX IF NOT EXISTS idx_flowai_template_categories_is_active ON flowai_template_categories(is_active);
+            CREATE INDEX IF NOT EXISTS idx_flowai_template_categories_order_index ON flowai_template_categories(order_index);
+
+            -- 템플릿 인덱스
+            CREATE INDEX IF NOT EXISTS idx_flowai_templates_category_id ON flowai_templates(category_id);
+            CREATE INDEX IF NOT EXISTS idx_flowai_templates_is_public ON flowai_templates(is_public);
+            CREATE INDEX IF NOT EXISTS idx_flowai_templates_user_id ON flowai_templates(user_id);
+            CREATE INDEX IF NOT EXISTS idx_flowai_templates_rating ON flowai_templates(rating);
+            CREATE INDEX IF NOT EXISTS idx_flowai_templates_usage_count ON flowai_templates(usage_count);
+            CREATE INDEX IF NOT EXISTS idx_flowai_templates_created_at ON flowai_templates(created_at);
+
+            -- 템플릿 리뷰 인덱스
+            CREATE INDEX IF NOT EXISTS idx_flowai_template_reviews_template_id ON flowai_template_reviews(template_id);
+            CREATE INDEX IF NOT EXISTS idx_flowai_template_reviews_user_id ON flowai_template_reviews(user_id);
+            CREATE INDEX IF NOT EXISTS idx_flowai_template_reviews_rating ON flowai_template_reviews(rating);
+
+            -- 템플릿 사용 통계 인덱스
+            CREATE INDEX IF NOT EXISTS idx_flowai_template_usage_template_id ON flowai_template_usage(template_id);
+            CREATE INDEX IF NOT EXISTS idx_flowai_template_usage_user_id ON flowai_template_usage(user_id);
+            CREATE INDEX IF NOT EXISTS idx_flowai_template_usage_used_at ON flowai_template_usage(used_at);
         `);
 
     // AI 피드백 관련 인덱스
@@ -1580,7 +1759,7 @@ const createTables = async (dbManager) => {
         console.warn('이 설정은 수동으로 적용해야 할 수 있습니다.');
       }
     }
-    console.log('데이터베이스 마이그레이션 완료 (버전 ' + newCurrentVersion + ' - PR #265 통합 및 스키마 안정성 개선)');
+    console.log('데이터베이스 마이그레이션 완료 (버전 ' + newCurrentVersion + ' - LLM 파인튜닝 시스템 추가)');
   } catch (error) {
     console.error('데이터베이스 마이그레이션 중 오류 발생:', error);
     await dbManager.query('ROLLBACK');
